# Function used for generating data


sptvpsv_reg_dgp <- function (Tt, p, param){
  
# ####################################################################
#           Basic Descriptions Here
# ####################################################################
  # print(nargs)
  
  if (nargs()==0){
    Tt <- 200
    p <- 8
    
    s <- matrix(.0,Tt,p)
    s[, c(1,3,5,7)] <- 1
    
    V0 <- log(.2)
    theta0 <- 2*(4*runif(p)+1)
    mu <- s[1, ]*theta0/2
    
    
  } else{
    s <- param[['s']]
    V0 <- log(param[['sigma0']])
    theta0 <- param[['theta0']]
    mu <- param[['mu']]
  }
  
  ## Generate sparse time-varying parameter model 
  
  # 1. First generate predictors
  x <- rnorm(Tt*p) %>% matrix(Tt,p)
  
  # 2. Generate unrestricted regression coefficients theta_t
  theta_t <- matrix(.0,Tt+100,p)
  
  for (t in seq(1,Tt+100)){
    if (t == 1){
      theta_t[t,] <- 0.9999*theta0 + (1/(Tt^(2/4))) * rnorm(p)
    } else{
      theta_t[t,] <- mu + 0.9999*(theta_t[t-1,]-mu) + (1/(Tt^(2/4)))*rnorm(p)
    }
  }
  theta_t <- theta_t[101:(Tt+100),]
  
  # 3. Define sparse (restricted) regression coefficients beta_t
  beta_t <- s * theta_t
  
  # 4. Generate unrestricted regression coefficients sigma_t
  V_t <- matrix(.0,Tt+100,1)
  
  for (t in seq(1,Tt+100)){
    if (t==1){
      V_t[t,] <- 0.99*V0 + (1/(Tt^(2/4)))*rnorm(1)
    } else{
      V_t[t,] <- V0 + 0.99*(V_t[t-1,] - V0) + (1/(Tt^(2/4)))*rnorm(1)
    }
  }
  
  sigma_t <- exp(V_t[101:(Tt+100),])
  
  # 5. Generate dependent variable y
  y <- rowSums(x*beta_t) + sqrt(sigma_t)*rnorm(Tt)
  
  return(list(
    y=y,
    x=x,
    s=s,
    theta_t=theta_t,
    beta_t=beta_t,
    sigma_t=sigma_t
  ))
}