\name{sptvpsv_reg_dgp}
\alias{sptvpsv_reg_dgp}
\title{Function used for generating data to be used}
\usage{
dgp_components <- sptvpsv_reg_dgp(Tt,p,param)
}
\arguments{
\item{Tt}{Length of Time-series observation, i.e. the number of rows of design matrix.}

\item{p}{How many covariates, i.e. the number of columns of design matrix.}

\item{param}{Necessary parameters specifying the DGP, it is a list with parameters contained in each field of the list.}

}
\value{
Returns a list with 6 components: \describe{

\item{\code{y}}{Response on the Left handside}

\item{\code{x}}{Covariates on the Right handside}

\item{\code{s}}{Indicator variable indicating whether the corresponding coefficients mattter
for the data generating process.}

\item{\code{theta_t}}{Underlying coefficient specifying the dynamic mechanism of time
varying parameters.}

\item{\code{beta_t}}{Time-varying parameters.}

\item{\code{sigma_t}}{Stochastic volatilities.}

}
}
\description{
This function will generate both response \code{y} and covariates \code{x}, along with the time-varying parameters and stochastic volatilities under some specilly designed mechanism.
}
\examples{
## Basic specification for Monte Carlo Experiment

Tt  <- 200          # Number of observations to generate
p   <- 200          # Number of predictors to generate

s <- matrix(1,Tt,p)
s[round(Tt/3):Tt,1] <- 0
s[round(Tt/2):Tt,3] <- 0
s[1:round(Tt/2),4] <- 0
s[, 5:p] <- 0

theta0 <- c(-1.7, 2.9, 1.4, -2.3, rep(0,p-4))
mu <- s[1, ] * theta0

param <- list()
param[['s']] <- s
param[['theta0']] <- theta0
param[['sigma0']] <- .1
param[['mu']] <- mu

# Generating data
dgp_components <- sptvpsv_reg_dgp(Tt,p,param)

}
