library(magrittr)
library(vbdvsarmadillo)

# source("sptvpsv_reg_dgp.R")

nMC <- 100          # Number of Monte Carlo iterations
Tt  <- 200          # Number of observations to generate
p   <- 200          # Number of predictors to generate

s <- matrix(1,Tt,p)
s[round(Tt/3):Tt,1] <- 0
s[round(Tt/2):Tt,3] <- 0
s[1:round(Tt/2),4] <- 0
s[, 5:p] <- 0

theta0 <- c(-1.7, 2.9, 1.4, -2.3, rep(0,p-4))
mu <- s[1, ] * theta0

param <- list()
param[['s']] <- s
param[['theta0']] <- theta0
param[['sigma0']] <- .1
param[['mu']] <- mu


# Storage
BETA <- array(.0,c(nMC,Tt,p,4))
SIGMA <- array(.0,c(nMC,Tt,4))
TVPROB <- array(.0,c(nMC,Tt,p))
Etimes <- array(.0,c(nMC,3))


# dgp_components <- sptvpsv_reg_dgp(Tt,p,param)
# y <- dgp_components$y
# x <- dgp_components$x
# s_sim <- dgp_components$s
# beta_sim <- dgp_components$beta_t
# sigma_sim <- dgp_components$sigma_t
# 
# theta_sim <- dgp_components$theta_t


# Estimate

set.seed(0)

for (iMC in seq(1,nMC)){
  
  dgp_components <- sptvpsv_reg_dgp(Tt,p,param)
  y <- dgp_components$y
  x <- dgp_components$x
  s_sim <- dgp_components$s
  beta_sim <- dgp_components$beta_t
  sigma_sim <- dgp_components$sigma_t
  theta_sim <- dgp_components$theta_t
  
  
  Tt <- dim(x)[1]
  p <- dim(x)[2]
  
  BETA[iMC, , , 1] <- beta_sim
  SIGMA[iMC, , 1] <- sigma_sim
  

  tic <- Sys.time()
  res <- VBDVS_armadillo(list(
    y=y,
    x=x,
    maxiter=100,
    prior=2,
    q=0,
    delta=0.9,
    c0=25,
    h0=12,
    g0=1
  ))
  toc <- Sys.time()
  
  toc - tic
  
  
  BETA[iMC, , , 2] <- res$beta
  SIGMA[iMC, , 2] <- res$sigma_t %>% as.vector()
  TVPROB[iMC, , ] <- res$tv_probs
  
  cat(sprintf("Done for %d-th Simulation\n",iMC))
  
}


probs <- apply(TVPROB,c(2,3),mean)
beta_median <- apply(BETA,c(2,3,4),median)
beta_high <- apply(BETA,c(2,3,4),quantile,probs=0.84,names=F)
beta_low <- apply(BETA,c(2,3,4),quantile,probs=0.16,names=F)

