// [[Rcpp::depends(RcppArmadillo)]]
#include <RcppArmadillo.h>

using namespace Rcpp;
using namespace arma;


// [[Rcpp::export]]
SEXP first_check(List input_params){
  mat A(as<mat>(input_params["A"]));
  int m = A.n_rows;
  int n =A.n_cols;
  // vec B;
  for (int i=0;i<m;i++){
    for (int j=0;j<n;j++){
      //A(i,j) = R::rnorm(0,1);
      //A.row(i) = as<vec>(rnorm(n)).t();
      A.row(i)=as<rowvec>(rnorm(n));
    }
  }
  // B = R::rowSums(A);
  return wrap(A);
}


// [[Rcpp::export]]
SEXP armadillo_mat_mult(List input_params){
  mat A(as<mat>(input_params["A"]));
  mat B(as<mat>(input_params["B"]));
  
  mat C;
  C = A*B;
  return wrap(C);
}


// [[Rcpp::export]]
SEXP armadillo_mat_add(List input_params){
  mat A(as<mat>(input_params["A"]));
  mat B(as<mat>(input_params["B"]));
  
  mat C;
  C = A+B;
  return wrap(exp(C));
}


// [[Rcpp::export]]
SEXP armadillo_cube(List input_params){
  int m(input_params["m"]);
  int n(input_params["n"]);
  int q(input_params["q"]);
  
  cube Y(m,n,q);
  Y.fill(0);
  Y.slice(0) = ones<mat>(m,n);
  Y.slice(1) = diagmat(as<vec>(rnorm(m)));
  
  return wrap(Y);
}

// [[Rcpp::export]]
arma::vec lnormpdf(const arma::vec& x, const arma::vec& mu, const arma::vec& sigma){
  //Constants
  static const double pi = 3.141592653589793238462643383280;
  
  NumericVector sigma_numvec(wrap(sigma));
  NumericVector mu_numvec(wrap(mu));
  NumericVector x_numvec(wrap(x));
  
  vec y;
  
  LogicalVector negative_index;
  negative_index = sigma_numvec <= 0;
  
  sigma_numvec[negative_index] = R_NaN;
  
  vec sigma_vec = as<vec>(sigma_numvec);
  vec mu_vec = as<vec>(mu_numvec);
  vec x_vec = as<vec>(x_numvec);
  
  y = log(1/(sqrt(2*pi)*sigma_vec)) - 0.5*square((x_vec-mu_vec)/sigma_vec);
  
  
  return y;
  
}

// [[Rcpp::export]]

SEXP const_mult_mat(List input_params){
  mat A(as<mat>(input_params["A"]));
  double C(input_params["C"]);
  
  return wrap(C*A);
}


// [[Rcpp::export]]
SEXP ld_gamma(List input_params){
  mat x(as<vec>(input_params["x"]));
  
  NumericVector num_x(wrap(x));
  
  NumericVector num_l_x = lgamma(num_x);
  NumericVector num_d_x = digamma(num_x);
  
  List output =  List::create(
    Named("lgammax")=num_l_x,
    Named("dgammax")=num_d_x
  );
  
  return wrap(output);
  
}


// [[Rcpp::export]]
double armadillo_norm(const arma::vec& x){
  return (norm(x,2));
}

// [[Rcpp::export]]
SEXP armadillo_mat_vec(const arma::mat& A){
  vec B;
  vec C = zeros<vec>(5);
  
  B = A*A.col(0);
  mat D = ones<mat>(5,3);
  
  List output = List::create(
    Named("vecB")=B,
    Named("vecC")=C,
    Named("matD")=D
  );
  return wrap(output);
}

// [[Rcpp::export]]
double scalar_power(const double& x){
  return (pow(x,2));
}

// [[Rcpp::export]]
arma::mat diag_from_mat(const arma::mat& x){
  return (diagmat(x.row(0)));
}


// [[Rcpp::export]]
arma::mat mat_txt(arma::cube x){
  // It seems that we cannot parse cube using "pointer" directly
  x.slice(3) = x.slice(0)*x.slice(1)*x.slice(2).t();
  return(x.slice(3));
}

// [[Rcpp::export]]
SEXP scalar_vec(const arma::mat& x){
  double coef;
  //mat c;
  //c = x.row(0)*x.col(1)+1;
  //coef = c(0,0);
  coef = dot(x.row(0),x.col(1))+1;
  // return(x.col(0)/coef);
  return wrap(x.col(0)/coef);
}

// [[Rcpp::export]]
arma::mat const_to_vec(arma::mat x){
  x.row(0).fill(3.5);
  x.row(0) = x.row(0)+1;
  return(x);
}

// [[Rcpp::export]]
arma::mat armadillo_sqrt(const arma::mat& x){
  return(sqrt(x));
  
}

// [[Rcpp::export]]
arma::vec armadillo_col_row(const arma::mat& x){
  // we need necessary transpose to ensure the dimension match
  return(1/(x.col(0)%x.row(1).t()));
}

// [[Rcpp::export]]
arma::mat armadillo_slicing_mat(arma::mat x){
  
  x(span(1,1),span(0,2)).fill(0);
  
  return(x);
  
}

// [[Rcpp::export]]
arma::vec klgamma(arma::mat pa,arma::mat pb,const double& qa, const double& qb){
  int pa_r = pa.n_rows;
  int pa_n = pa.n_cols;
  int pb_r = pb.n_rows;
  int pb_n = pb.n_cols;
  int n = std::max(pa_n,pb_n);
  int vec_length = pa_r*n;
  
  mat nx = ones<mat>(pa_r,n);
  vec qavec = vectorise(qa*nx);
  vec qbvec = vectorise(qb*nx);
  
   
  
  //if (pa_n==1){
  //  mat pa_new = pa*nx;
  //  return(pa_new);
  //} else{
  //  return(pa);
  //}
  
  
  if (pa_n==1){
    pa=pa*ones<rowvec>(n);
  }
  if (pb_n==1){
    pb=pb*ones<rowvec>(n);
  }
  
  NumericVector pa_vec(wrap(vectorise(pa)));
  NumericVector pb_vec(wrap(vectorise(pb)));
  NumericVector qa_vec(wrap(qavec));
  NumericVector qb_vec(wrap(qbvec));
  
  NumericVector temp_num(vec_length); 
  
  temp_num = pa_vec*Rcpp::log(pb_vec)-lgamma(pa_vec)
    -qa_vec*log(qb_vec)+lgamma(qa_vec)
    +(pa_vec-qa_vec)*(digamma(pa_vec)-Rcpp::log(pb_vec))
    -(pb_vec-qb_vec)*pa_vec/pb_vec;
    
  vec temp_vec(as<vec>(temp_num));
  mat temp_mat(temp_vec);
  temp_mat.reshape(pa_r,n);
  
  vec row_sum_vec = sum(temp_mat,1);
  return(row_sum_vec);
  
}


// [[Rcpp::export]]
SEXP ld_gamma_mat(List input_params){
  vec x(as<vec>(input_params["x"]));
  
  NumericVector num_x(wrap(x));
  
  NumericVector num_l_x = lgamma(num_x);
  NumericVector num_d_x = digamma(num_x);
  
  vec armavec_l(as<vec>(num_l_x));
  vec armavec_d(as<vec>(num_d_x));
  mat armamat_l = diagmat(armavec_l);
  
  List output =  List::create(
    Named("lgammax")=armavec_l,
    Named("dgammax")=armavec_d,
    Named("armamat")=armamat_l
  );
  
  return wrap(output);
  
}


// [[Rcpp::export]]
arma::mat armadillo_reshape(arma::mat x){
  vec vec_x = vectorise(x);
  NumericVector num_vec_x(wrap(vec_x));
  
  NumericVector numvec_l = lgamma(num_vec_x);
  vec vec_l(as<vec>(numvec_l));
  mat vec_mat(vec_l);
  vec_mat.reshape(3,3);
  
  return(vec_mat);
  
}

// [[Rcpp::export]]
NumericVector Rcpp_log(const NumericVector& x){
  return(Rcpp::log(x));
}


// [[Rcpp::export]]
NumericVector numericvector_txt(const NumericMatrix& x){
  NumericVector v;
  v = x(_,0)*x(_,1)/x(_,2);
  return(v);
}
